@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# pylint: disable=unsubscriptable-object,ungrouped-imports,line-too-long
 """URL Utilities
 ===================
 
@@ -18,20 +19,25 @@ import functools
 import hashlib
 import os
 import re
-import urllib.parse
+import urllib.parse as urllib_parse
+from typing import TYPE_CHECKING
 
-import darc.typing as typing
 from darc.const import PATH_DB
 
 try:
     from pathlib import PosixPath
     PosixPath(os.curdir)
 except NotImplementedError:
-    from pathlib import PurePosixPath as PosixPath  # type: ignore
+    from pathlib import PurePosixPath as PosixPath  # type: ignore[misc]
+
+if TYPE_CHECKING:
+    from typing import Any, AnyStr, Dict, Optional, Union
+    from urllib.parse import ParseResult
+
+    _Str = Union[bytes, str]
 
 
-def quote(string: typing.AnyStr, safe: typing.AnyStr = '/',  # type: ignore
-          encoding: typing.Optional[str] = None, errors: typing.Optional[str] = None) -> str:
+def quote(string: str, safe: '_Str' = '/', encoding: 'Optional[str]' = None, errors: 'Optional[str]' = None) -> str:
     """Wrapper function for :func:`urllib.parse.quote`.
 
     Args:
@@ -50,11 +56,11 @@ def quote(string: typing.AnyStr, safe: typing.AnyStr = '/',  # type: ignore
 
     """
     with contextlib.suppress(Exception):
-        return urllib.parse.quote(string, safe, encoding=encoding, errors=errors)  # type: ignore
+        return urllib_parse.quote(string, safe, encoding=encoding, errors=errors)
     return str(string)
 
 
-def unquote(string: typing.AnyStr, encoding: str = 'utf-8', errors: str = 'replace') -> str:
+def unquote(string: str, encoding: str = 'utf-8', errors: str = 'replace') -> str:
     """Wrapper function for :func:`urllib.parse.unquote`.
 
     Args:
@@ -72,11 +78,11 @@ def unquote(string: typing.AnyStr, encoding: str = 'utf-8', errors: str = 'repla
 
     """
     with contextlib.suppress(Exception):
-        return urllib.parse.unquote(string, encoding=encoding, errors=errors)  # type: ignore
+        return urllib_parse.unquote(string, encoding=encoding, errors=errors)
     return str(string)
 
 
-def urljoin(base: typing.AnyStr, url: typing.AnyStr, allow_fragments: bool = True) -> str:
+def urljoin(base: 'AnyStr', url: 'AnyStr', allow_fragments: bool = True) -> 'AnyStr':
     """Wrapper function for :func:`urllib.parse.urljoin`.
 
     Args:
@@ -94,11 +100,13 @@ def urljoin(base: typing.AnyStr, url: typing.AnyStr, allow_fragments: bool = Tru
 
     """
     with contextlib.suppress(ValueError):
-        return urllib.parse.urljoin(base, url, allow_fragments=allow_fragments)  # type: ignore
-    return f'{str(base)}/{str(url)}'
+        return urllib_parse.urljoin(base, url, allow_fragments=allow_fragments)
+    if isinstance(base, bytes):
+        return b'%s/%s' % (base, url)
+    return f'{base}/{url}'
 
 
-def urlparse(url: str, scheme: str = '', allow_fragments: bool = True) -> urllib.parse.ParseResult:
+def urlparse(url: str, scheme: str = '', allow_fragments: bool = True) -> 'ParseResult':
     """Wrapper function for :func:`urllib.parse.urlparse`.
 
     Args:
@@ -117,11 +125,11 @@ def urlparse(url: str, scheme: str = '', allow_fragments: bool = True) -> urllib
 
     """
     with contextlib.suppress(ValueError):
-        return urllib.parse.urlparse(url, scheme, allow_fragments=allow_fragments)
-    return urllib.parse.ParseResult(scheme=scheme, netloc='', path=url, params='', query='', fragment='')
+        return urllib_parse.urlparse(url, scheme, allow_fragments=allow_fragments)
+    return urllib_parse.ParseResult(scheme=scheme, netloc='', path=url, params='', query='', fragment='')
 
 
-def urlsplit(url: str, scheme: str = '', allow_fragments: bool = True) -> urllib.parse.SplitResult:
+def urlsplit(url: str, scheme: str = '', allow_fragments: bool = True) -> urllib_parse.SplitResult:
     """Wrapper function for :func:`urllib.parse.urlsplit`.
 
     Args:
@@ -140,8 +148,8 @@ def urlsplit(url: str, scheme: str = '', allow_fragments: bool = True) -> urllib
 
     """
     with contextlib.suppress(ValueError):
-        return urllib.parse.urlsplit(url, scheme, allow_fragments=allow_fragments)
-    return urllib.parse.SplitResult(scheme=scheme, netloc='', path=url, query='', fragment='')
+        return urllib_parse.urlsplit(url, scheme, allow_fragments=allow_fragments)
+    return urllib_parse.SplitResult(scheme=scheme, netloc='', path=url, query='', fragment='')
 
 
 @dataclasses.dataclass
@@ -156,6 +164,8 @@ class Link:
         base: base folder for saving files
         name: hashed link for saving files
         url_parse: parsed URL from :func:`urllib.parse.urlparse`
+        url_backref: optional :class:`~darc.link.Link` instance
+            from which current link was extracted
 
     Returns:
         :class:`~darc.link.Link`: Parsed link object.
@@ -173,16 +183,19 @@ class Link:
     #: proxy type
     proxy: str
 
-    #: parsed URL from :func:`urllib.parse.urlparse`
-    url_parse: urllib.parse.ParseResult
-
     #: URL's hostname
-    host: str
+    host: 'Optional[str]'
     #: base folder for saving files
     base: str
     #: hashed link for saving files
     name: str
 
+    #: parsed URL from :func:`urllib.parse.urlparse`
+    url_parse: urllib_parse.ParseResult
+    #: optional :class:`~darc.link.Link` instance
+    #: from which current link was extracted
+    url_backref: 'Optional[Link]' = None
+
     def __hash__(self) -> int:
         """Provide hash support to the :class:`~darc.link.Link` object."""
         return hash(self.url)
@@ -190,24 +203,39 @@ class Link:
     def __str__(self) -> str:
         return self.url
 
-    def __eq__(self, value: typing.Any) -> bool:
+    def __eq__(self, value: 'Any') -> bool:
         if isinstance(value, Link):
             return self.url == value.url
         return False
 
-    def __lt__(self, value: typing.Any) -> bool:
+    def __lt__(self, value: 'Any') -> bool:
         if isinstance(value, Link):
             return self.url < value.url
         raise TypeError(f"'<' not supported between instances of 'Link' and {type(value).__name__!r}")
 
+    def asdict(self) -> 'Dict[str, Any]':
+        """Convert to a :obj:`dict` instance."""
+        return {
+            'url': self.url,
+            'proxy': self.proxy,
+            'host': self.host,
+            'base': os.path.relpath(self.base, PATH_DB),
+            'name': self.name,
+            'backref': backref.url if (backref := self.url_backref) is not None else None,  # pylint: disable=used-before-assignment
+        }
 
-def parse_link(link: str, host: typing.Optional[str] = None) -> Link:
+
+def parse_link(link: str, host: 'Optional[str]' = None, *, backref: 'Optional[Link]' = None) -> 'Link':
     """Parse link.
 
     Args:
         link: link to be parsed
         host: hostname of the link
 
+    Keyword Args:
+        backref: optional :class:`~darc.link.Link` instance
+            from which current link was extracted
+
     Returns:
         The parsed link object.
 
@@ -226,48 +254,50 @@ def parse_link(link: str, host: typing.Optional[str] = None) -> Link:
         JavaScript codes, set ``proxy`` as ``script``.
     3.  If the scheme is ``bitcoin``, then the link is a Bitcoin
         address, set ``proxy`` as ``bitcoin``.
-    4.  If the scheme is ``ed2k``, then the link is an ED2K magnet
+    4.  If the scheme is ``ethereum``, then the link is an Ethereum
+        address, set ``proxy`` as ``ethereum``.
+    5.  If the scheme is ``ed2k``, then the link is an ED2K magnet
         link, set ``proxy`` as ``ed2k``.
-    5.  If the scheme is ``magnet``, then the link is a magnet
+    6.  If the scheme is ``magnet``, then the link is a magnet
         link, set ``proxy`` as ``magnet``.
-    6.  If the scheme is ``mailto``, then the link is an email
+    7.  If the scheme is ``mailto``, then the link is an email
         address, set ``proxy`` as ``mail``.
-    7.  If the scheme is ``irc``, then the link is an IRC
+    8.  If the scheme is ``irc``, then the link is an IRC
         link, set ``proxy`` as ``irc``.
-    8.  If the scheme is **NOT** any of ``http`` or ``https``,
+    9.  If the scheme is **NOT** any of ``http`` or ``https``,
         then set ``proxy`` to the scheme.
-    9.  If the host is :data:`None`, set ``hostname`` to ``(null)``,
+    10. If the host is :data:`None`, set ``hostname`` to ``(null)``,
         set ``proxy`` to ``null``.
-    10. If the host is an onion (``.onion``) address,
+    11. If the host is an onion (``.onion``) address,
         set ``proxy`` to ``tor``.
-    11. If the host is an I2P (``.i2p``) address, or
+    12. If the host is an I2P (``.i2p``) address, or
         any of ``localhost:7657`` and ``localhost:7658``,
         set ``proxy`` to ``i2p``.
-    12. If the host is *localhost* on :data:`~darc.proxy.zeronet.ZERONET_PORT`,
+    13. If the host is *localhost* on :data:`~darc.proxy.zeronet.ZERONET_PORT`,
         and the path is not ``/``, i.e. **NOT** root path, set ``proxy``
         to ``zeronet``; and set the first part of its path as ``hostname``.
 
         Example:
 
-           For a ZeroNet address, e.g.
-           http://127.0.0.1:43110/1HeLLo4uzjaLetFx6NH3PMwFP3qbRbTf3D,
+           For a ZeroNet address, e.g.,
+           ``http://127.0.0.1:43110/1HeLLo4uzjaLetFx6NH3PMwFP3qbRbTf3D``,
            :func:`~darc.link.parse_link` will parse the ``hostname`` as
            ``1HeLLo4uzjaLetFx6NH3PMwFP3qbRbTf3D``.
 
-    13. If the host is *localhost* on :data:`~darc.proxy.freenet.FREENET_PORT`,
+    14. If the host is *localhost* on :data:`~darc.proxy.freenet.FREENET_PORT`,
         and the path is not ``/``, i.e. **NOT** root path, set ``proxy``
         to ``freenet``; and set the first part of its path as ``hostname``.
 
         Example:
 
-           For a Freenet address, e.g.
-           http://127.0.0.1:8888/USK@nwa8lHa271k2QvJ8aa0Ov7IHAV-DFOCFgmDt3X6BpCI,DuQSUZiI~agF8c-6tjsFFGuZ8eICrzWCILB60nT8KKo,AQACAAE/sone/77/,
+           For a Freenet address, e.g.,
+           ``http://127.0.0.1:8888/USK@nwa8lHa271k2QvJ8aa0Ov7IHAV-DFOCFgmDt3X6BpCI,DuQSUZiI~agF8c-6tjsFFGuZ8eICrzWCILB60nT8KKo,AQACAAE/sone/77/``,
            :func:`~darc.link.parse_link` will parse the ``hostname`` as
            ``USK@nwa8lHa271k2QvJ8aa0Ov7IHAV-DFOCFgmDt3X6BpCI,DuQSUZiI~agF8c-6tjsFFGuZ8eICrzWCILB60nT8KKo,AQACAAE``.
 
-    14. If the host is a proxied onion (``.onion.sh``) address,
+    15. If the host is a proxied onion (``.onion.sh``) address,
         set ``proxy`` to ``tor2web``.
-    15. If none of the cases above satisfied, the ``proxy`` will be set
+    16. If none of the cases above satisfied, the ``proxy`` will be set
         as ``null``, marking it a plain normal link.
 
     The ``base`` for parsed link :class:`~darc.link.Link` object is defined as
@@ -301,9 +331,12 @@ def parse_link(link: str, host: typing.Optional[str] = None) -> Link:
     elif scheme == 'javascript':
         proxy_type = 'script'
         host = '(script)'
-    elif scheme == 'bitcoin':
+    elif scheme in ('bitcoin', 'btc'):
         proxy_type = 'bitcoin'
         host = '(bitcoin)'
+    elif scheme in ('ethereum', 'eth'):
+        proxy_type = 'ethereum'
+        host = '(ethereum)'
     elif scheme == 'ed2k':
         proxy_type = 'ed2k'
         host = '(ed2k)'
@@ -356,14 +389,15 @@ def parse_link(link: str, host: typing.Optional[str] = None) -> Link:
     else:
         proxy_type = 'null'
 
-    # <proxy>/<scheme>/<host>/<hash>-<timestamp>.html
+    # <proxy>/<scheme>/<hostname>/<hash>-<timestamp>.html
     base = os.path.join(PATH_DB, proxy_type, scheme, hostname)
     name = hashlib.sha256(link.encode()).hexdigest()
 
     return Link(
         url=link,
         url_parse=parse,
-        host=host,  # type: ignore
+        url_backref=backref,
+        host=host,
         base=base,
         name=name,
         proxy=proxy_type,
