@@ -12,24 +12,16 @@ to store the data crawled from the :mod:`darc` project.
 
 """
 
-from darc.const import DB_WEB as DB
 from darc.model.web.hostname import HostnameModel
 from darc.model.web.hosts import HostsModel
-from darc.model.web.requests import RequestsModel, RequestsHistoryModel
+from darc.model.web.requests import RequestsHistoryModel, RequestsModel
 from darc.model.web.robots import RobotsModel
 from darc.model.web.selenium import SeleniumModel
 from darc.model.web.sitemap import SitemapModel
-from darc.model.web.url import URLModel
+from darc.model.web.url import URLModel, URLThroughModel
 
 __all__ = [
-    'HostnameModel', 'URLModel',
+    'HostnameModel', 'URLModel', 'URLThroughModel',
     'RobotsModel', 'SitemapModel', 'HostsModel',
     'RequestsModel', 'RequestsHistoryModel', 'SeleniumModel',
 ]
-
-with DB:
-    DB.create_tables([
-        HostnameModel, URLModel,
-        RobotsModel, SitemapModel, HostsModel,
-        RequestsModel, RequestsHistoryModel, SeleniumModel,
-    ])
