@@ -11,19 +11,20 @@ addresses extracted to the data storage file
 
 """
 
+import json
 import os
 from typing import TYPE_CHECKING
 
 from darc.const import PATH_MISC, get_lock
 
 if TYPE_CHECKING:
-    from darc.link import Link
+    import darc.link as darc_link  # Link
 
 PATH = os.path.join(PATH_MISC, 'mail.txt')
 LOCK = get_lock()
 
 
-def save_mail(link: 'Link') -> None:
+def save_mail(link: 'darc_link.Link') -> None:
     """Save email address.
 
     The function will save email address to the file
@@ -35,4 +36,7 @@ def save_mail(link: 'Link') -> None:
     """
     with LOCK:  # type: ignore[union-attr]
         with open(PATH, 'a') as file:
-            print(link.url, file=file)
+            print(json.dumps({
+                'src': backref.url if (backref := link.url_backref) is not None else None,  # pylint: disable=used-before-assignment
+                'url': link.url,
+            }), file=file)
