@@ -9,22 +9,18 @@ around managing and processing normal websites with no proxy.
 
 import gzip
 import io
+import json
 import os
-import sys
 from typing import TYPE_CHECKING
 
 import bs4
 import requests
-import stem
-import stem.control
-import stem.process
-import stem.util.term
 
 from darc._compat import RobotFileParser
 from darc.const import CHECK, PATH_MISC, get_lock
 from darc.db import save_requests
-from darc.error import render_error
 from darc.link import parse_link
+from darc.logging import logger
 from darc.parse import _check, get_content_type, urljoin
 from darc.requests import request_session
 from darc.save import save_link
@@ -32,13 +28,13 @@ from darc.save import save_link
 if TYPE_CHECKING:
     from typing import List, Optional
 
-    from darc.link import Link
+    import darc.link as darc_link  # Link
 
 PATH = os.path.join(PATH_MISC, 'invalid.txt')
 LOCK = get_lock()
 
 
-def save_invalid(link: 'Link') -> None:
+def save_invalid(link: 'darc_link.Link') -> None:
     """Save link with invalid scheme.
 
     The function will save link with invalid scheme to the file
@@ -48,12 +44,15 @@ def save_invalid(link: 'Link') -> None:
         link: Link object representing the link with invalid scheme.
 
     """
-    with LOCK:  # type: ignore
+    with LOCK:  # type: ignore[union-attr]
         with open(PATH, 'a') as file:
-            print(link.url, file=file)
+            print(json.dumps({
+                'src': backref.url if (backref := link.url_backref) is not None else None,  # pylint: disable=used-before-assignment
+                'url': link.url,
+            }), file=file)
 
 
-def save_robots(link: 'Link', text: str) -> str:
+def save_robots(link: 'darc_link.Link', text: str) -> str:
     """Save ``robots.txt``.
 
     Args:
@@ -79,7 +78,7 @@ def save_robots(link: 'Link', text: str) -> str:
     return path
 
 
-def save_sitemap(link: 'Link', text: str) -> str:
+def save_sitemap(link: 'darc_link.Link', text: str) -> str:
     """Save sitemap.
 
     Args:
@@ -108,7 +107,7 @@ def save_sitemap(link: 'Link', text: str) -> str:
     return path
 
 
-def have_robots(link: 'Link') -> 'Optional[str]':
+def have_robots(link: 'darc_link.Link') -> 'Optional[str]':
     """Check if ``robots.txt`` already exists.
 
     Args:
@@ -125,7 +124,7 @@ def have_robots(link: 'Link') -> 'Optional[str]':
     return path if os.path.isfile(path) else None
 
 
-def have_sitemap(link: 'Link') -> 'Optional[str]':
+def have_sitemap(link: 'darc_link.Link') -> 'Optional[str]':
     """Check if sitemap already exists.
 
     Args:
@@ -142,7 +141,7 @@ def have_sitemap(link: 'Link') -> 'Optional[str]':
     return path if os.path.isfile(path) else None
 
 
-def read_robots(link: 'Link', text: str, host: 'Optional[str]' = None) -> 'List[Link]':
+def read_robots(link: 'darc_link.Link', text: str, host: 'Optional[str]' = None) -> 'List[darc_link.Link]':
     """Read ``robots.txt`` to fetch link to sitemaps.
 
     Args:
@@ -172,7 +171,7 @@ def read_robots(link: 'Link', text: str, host: 'Optional[str]' = None) -> 'List[
     return [parse_link(urljoin(link.url, sitemap), host=host, backref=link) for sitemap in sitemaps]
 
 
-def get_sitemap(link: 'Link', text: str, host: 'Optional[str]' = None) -> 'List[Link]':
+def get_sitemap(link: 'darc_link.Link', text: str, host: 'Optional[str]' = None) -> 'List[darc_link.Link]':
     """Fetch link to other sitemaps from a sitemap.
 
     Args:
@@ -200,7 +199,7 @@ def get_sitemap(link: 'Link', text: str, host: 'Optional[str]' = None) -> 'List[
     return [parse_link(sitemap, host=host, backref=link) for sitemap in sitemaps]
 
 
-def read_sitemap(link: 'Link', text: str, check: bool = CHECK) -> 'List[Link]':
+def read_sitemap(link: 'darc_link.Link', text: str, check: bool = CHECK) -> 'List[darc_link.Link]':
     """Read sitemap.
 
     Args:
@@ -229,7 +228,7 @@ def read_sitemap(link: 'Link', text: str, check: bool = CHECK) -> 'List[Link]':
     return temp_list
 
 
-def fetch_sitemap(link: 'Link', force: bool = False) -> None:
+def fetch_sitemap(link: 'darc_link.Link', force: bool = False) -> None:
     """Fetch sitemap.
 
     The function will first fetch the ``robots.txt``, then
@@ -249,74 +248,65 @@ def fetch_sitemap(link: 'Link', force: bool = False) -> None:
 
     """
     if force:
-        print(stem.util.term.format(f'[ROBOTS] Force refetch {link.url}',
-                                    stem.util.term.Color.YELLOW))  # pylint: disable=no-member
+        logger.warning('[ROBOTS] Force refetch %s', link.url)
 
     robots_path = None if force else have_robots(link)
     if robots_path is not None:
 
-        print(stem.util.term.format(f'[ROBOTS] Cached {link.url}',
-                                    stem.util.term.Color.YELLOW))  # pylint: disable=no-member
+        logger.warning('[ROBOTS] Cached %s', link.url)
         with open(robots_path) as file:
             robots_text = file.read()
 
     else:
 
         robots_link = parse_link(urljoin(link.url, '/robots.txt'), backref=link)
-        print(f'[ROBOTS] Checking {robots_link.url}')
+        logger.info('[ROBOTS] Checking %s', robots_link.url)
 
         with request_session(robots_link) as session:
             try:
                 response = session.get(robots_link.url)
-            except requests.RequestException as error:
-                print(render_error(f'[ROBOTS] Failed on {robots_link.url} <{error}>',
-                                   stem.util.term.Color.RED), file=sys.stderr)  # pylint: disable=no-member
+            except requests.RequestException:
+                logger.pexc(message=f'[ROBOTS] Failed on {robots_link.url}')
                 return
 
         if response.ok:
             ct_type = get_content_type(response)
             if ct_type not in ['text/text', 'text/plain']:
-                print(render_error(f'[ROBOTS] Unresolved content type on {robots_link.url} ({ct_type})',
-                                   stem.util.term.Color.RED), file=sys.stderr)  # pylint: disable=no-member
+                logger.error('[ROBOTS] Unresolved content type on %s (%s)', robots_link.url, ct_type)
                 robots_text = ''
             else:
                 robots_text = response.text
                 save_robots(robots_link, robots_text)
-                print(f'[ROBOTS] Checked {robots_link.url}')
+                logger.info('[ROBOTS] Checked %s', robots_link.url)
         else:
-            print(render_error(f'[ROBOTS] Failed on {robots_link.url} [{response.status_code}]',
-                               stem.util.term.Color.RED), file=sys.stderr)  # pylint: disable=no-member
+            logger.error('[ROBOTS] Failed on %s [%d]', robots_link.url, response.status_code)
             robots_text = ''
 
     if force:
-        print(stem.util.term.format(f'[SITEMAP] Force refetch {link.url}',
-                                    stem.util.term.Color.YELLOW))  # pylint: disable=no-member
+        logger.warning('[SITEMAP] Force refetch %s', link.url)
 
     sitemaps = read_robots(link, robots_text, host=link.host)
     for sitemap_link in sitemaps:
         sitemap_path = None if force else have_sitemap(sitemap_link)
         if sitemap_path is not None:
 
-            print(stem.util.term.format(f'[SITEMAP] Cached {sitemap_link.url}',
-                                        stem.util.term.Color.YELLOW))  # pylint: disable=no-member
+            logger.warning('[SITEMAP] Cached %s', sitemap_link.url)
             with open(sitemap_path) as file:
                 sitemap_text = file.read()
 
         else:
 
-            print(f'[SITEMAP] Fetching {sitemap_link.url}')
+            logger.info('[SITEMAP] Fetching %s', sitemap_link.url)
 
             with request_session(sitemap_link) as session:
                 try:
                     response = session.get(sitemap_link.url)
-                except requests.RequestException as error:
-                    print(render_error(f'[SITEMAP] Failed on {sitemap_link.url} <{error}>',
-                                       stem.util.term.Color.RED), file=sys.stderr)  # pylint: disable=no-member
+                except requests.RequestException:
+                    logger.pexc(message=f'[SITEMAP] Failed on {sitemap_link.url}')
                     continue
 
             if not response.ok:
-                print(render_error(f'[SITEMAP] Failed on {sitemap_link.url} [{response.status_code}]',
-                                   stem.util.term.Color.RED), file=sys.stderr)  # pylint: disable=no-member
+                logger.error('[SITEMAP] Failed on %s [%d]', sitemap_link.url, response.status_code)
                 continue
 
             # check content type
@@ -330,11 +320,10 @@ def fetch_sitemap(link: 'Link', force: bool = False) -> None:
                 sitemap_text = response.text
                 save_sitemap(sitemap_link, sitemap_text)
             else:
-                print(render_error(f'[SITEMAP] Unresolved content type on {sitemap_link.url} ({ct_type})',
-                                   stem.util.term.Color.RED), file=sys.stderr)  # pylint: disable=no-member
+                logger.error('[SITEMAP] Unresolved content type on %s (%s)', sitemap_link.url, ct_type)
                 continue
 
-            print(f'[SITEMAP] Fetched {sitemap_link.url}')
+            logger.info('[SITEMAP] Fetched %s', sitemap_link.url)
 
         # get more sitemaps
         sitemaps.extend(get_sitemap(sitemap_link, sitemap_text, host=link.host))
