@@ -4,12 +4,16 @@
 
 import argparse
 import os
-import subprocess
+import subprocess  # nosec: B404
 import sys
 import tempfile
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from argparse import ArgumentParser
 
 
-def get_parser():
+def get_parser() -> 'ArgumentParser':
     """Argument parser."""
     parser = argparse.ArgumentParser('install',
                                      description='pseudo-interactive package installer')
@@ -20,18 +24,18 @@ def get_parser():
     return parser
 
 
-def main():
+def main() -> int:
     """Entrypoint."""
     parser = get_parser()
     args = parser.parse_args()
     text = args.stdin.encode().decode('unicode_escape')
 
-    path = tempfile.mktemp(prefix='install-')
+    path = tempfile.mktemp(prefix='install-')  # nosec: B306
     with open(path, 'w') as file:
         file.write(text)
 
     with open(path, 'r') as file:
-        proc = subprocess.run(args.command, stdin=file)  # pylint: disable=subprocess-run-check
+        proc = subprocess.run(args.command, stdin=file)  # pylint: disable=subprocess-run-check # nosec: B603
 
     os.remove(path)
     return proc.returncode
